package com.scenegraphshell;

import bsh.Interpreter;
import bsh.EvalError;

import com.sun.scenario.scenegraph.*;
import com.sun.scenario.animation.Timeline;

import javax.swing.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Set;
import java.util.HashSet;

/**
 *  Responsible for "Compiling" (or rather interpret) java code and create a scene graph structure out of it.
 */
public class ScenegraphCompiler implements Runnable{
    private Thread t = new Thread(this);
    private String lastExpression = "";
    private String thisExpression = "";
    private int updateInterval = 1000;
    private final ScenegraphEditor scenegraphEditor;
    private final ScenegraphPreviewComponent scenegraphPreviewComponent;

    private CompilerInterface compilerInterface;

    public ScenegraphCompiler(ScenegraphEditor scenegraphEditor, ScenegraphPreviewComponent scenegraphPreviewComponent){
        this.scenegraphEditor = scenegraphEditor;
        this.scenegraphPreviewComponent = scenegraphPreviewComponent;
        t.start();
    }

    public void setCompilerInterface(CompilerInterface compilerInterface) {
        this.compilerInterface = compilerInterface;
    }

    public void run() {
        while (t!=null){
            synchronized (t){
                try {
                    t.wait(updateInterval);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            try{
                if (scenegraphEditor.allowCompilation()){
                    SwingUtilities.invokeAndWait(new Runnable() {
                        public void run() {
                            thisExpression = scenegraphEditor.getCode();
                            if (!thisExpression.equals(lastExpression)){
                                scenegraphEditor.setStatus("Compiling",true,-1);
                            }
                        }
                    });
                }
            }
            catch (Exception e){
                e.printStackTrace();
            }
            if (!thisExpression.equals(lastExpression) && compilerInterface!=null){
                lastExpression = thisExpression;
                try {
                    compilerInterface.evaluate(lastExpression);
                    final SGNode node = compilerInterface.getSNNodeInstance();
                    final Timeline timeline = compilerInterface.getTimeline(); 
                    if (node!=null){
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                scenegraphPreviewComponent.setScene(node,timeline);
                                scenegraphEditor.setStatus("Compilation ok",false,-1);
                            }
                        });
                    }
                }
                catch (CompileError ce){
                    Set<Integer> lineNumbers = new HashSet<Integer>();
                    StringBuilder sb = new StringBuilder("<html>");
                    while (ce != null){
                        lineNumbers.add(ce.getLineNumber());
                        sb.append(ce.getErrorMessage());
                        sb.append("<br>");
                        Throwable e = ce.getCause();
                        if (e instanceof CompileError){
                            ce = (CompileError) e;
                        }
                        else{
                            ce = null;
                        }
                    }
                    scenegraphEditor.setStatus(sb.toString(), false, lineNumbers);
                }
                catch (Throwable t){
                    scenegraphEditor.setStatus("Error "+t.toString(),false,-1);
                }
            }
        }
    }
}