package com.scenegraphshell;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Set;

/**
 * Editor panel - responsible for displaying editor with lines and statusbar
 */
public class ScenegraphEditor extends JPanel {
    private JTextArea textArea = new JTextArea();
    private LineNumberBorder lineNumberBorder;
    private JLabel statusbar = new JLabel(" ");
    private JProgressBar progressBar = new JProgressBar();
    private JScrollPane scrollPane = new JScrollPane(textArea);
    private TitledBorder titledBorder = new TitledBorder("Editor");

    private JCheckBox autoCompile = new JCheckBox("Auto compile",true);
    private JButton compileButton = new JButton("Compile");
    private boolean compileNext = false;

    public ScenegraphEditor(String text) {
        setBorder(titledBorder);
        setLayout(new BorderLayout());

        textArea.setText(text);
        Font editorFont = textArea.getFont();
        editorFont = new Font(Font.MONOSPACED, editorFont.getStyle(), editorFont.getSize());
        textArea.setFont(editorFont);
        lineNumberBorder = new LineNumberBorder(textArea);

        scrollPane.setRowHeaderView(lineNumberBorder);
        JPanel progressbarPanel = new JPanel(new BorderLayout(5,5));
        progressbarPanel.add(statusbar, BorderLayout.CENTER);
        progressbarPanel.add(progressBar, BorderLayout.WEST);

        JPanel compilationAction = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        compilationAction.add(autoCompile);
        compilationAction.add(compileButton);
        compileButton.setEnabled(false);

        autoCompile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                compileButton.setEnabled(!autoCompile.isSelected());
            }
        });

        compileButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                compileNext = true; // mark next compile hit as true
            }
        });

        progressbarPanel.add(compilationAction,BorderLayout.EAST);

        add(scrollPane, BorderLayout.CENTER);
        add(progressbarPanel, BorderLayout.SOUTH);
    }

    public void setTitle(String title){
        titledBorder.setTitle(title);
    }

    public void setStatus(String text, boolean loading, Set<Integer> highlightLinenumber) {
        statusbar.setText(text);
        statusbar.revalidate(); // make sure that it fits
        statusbar.validate(); // make sure that it fits
        statusbar.repaint(); // make sure that it fits
        progressBar.setIndeterminate(loading);
        lineNumberBorder.setHighlightLinenumber(highlightLinenumber);
    }

    public void setStatus(String text, boolean loading, int highlightLinenumber) {
        statusbar.setText(text);
        statusbar.revalidate(); // make sure that it fits
        statusbar.validate(); // make sure that it fits
        statusbar.repaint(); // make sure that it fits
        progressBar.setIndeterminate(loading);
        lineNumberBorder.setHighlightLinenumber(highlightLinenumber);
    }

    public String getCode(){
        return textArea.getText();
    }

    public void setCode(String code) {
        textArea.setText(code);
        textArea.setCaretPosition(0);
        scrollPane.getViewport().setViewPosition(new Point(0,0));
    }

    public boolean allowCompilation(){
        boolean res = autoCompile.isSelected()||compileNext;
        compileNext = false;
        return res;
    }
}
